float4x4 worldViewProjI; 
uniform float time;
uniform float2 resolution = float2(933,511) ; // Screensize;

// An attempt at Quilez's warping (domain distortions):
// http://iquilezles.org/www/articles/warp/warp.htm
// 
// Not as good as his, but still interesting.
// @SyntopiaDK, 2012



float rand(float2 co){
	// implementation found at: lumina.sourceforge.net/Tutorials/Noise.html
	return frac(sin(dot(co.xy ,float2(12.9898,78.233))) * 43758.5453);   }




float noise2f( in float2 p )   {
	float2 ip = float2(floor(p));
	float2 u = frac(p);
	// http://www.iquilezles.org/www/articles/morenoise/morenoise.htm
	u = u*u*(3.0-2.0*u);
	//u = u*u*u*((6.0*u-15.0)*u+10.0);
	
	float res = lerp(
		lerp(rand(ip),  rand(ip+float2(1.0,0.0)),u.x),
		lerp(rand(ip+float2(0.0,1.0)),   rand(ip+float2(1.0,1.0)),u.x),
		u.y)
	;
	return res*res;
	//return 2.0* (res-0.5);
}



float fbm(float2 c) {
	float f = 0.0;
	float w = 1.0;
	for (int i = 0; i < 8; i++) {
		f+= w*noise2f(c);
		c*=1.0;
		w*=0.5;
	}
	return f;
}



float2 cMul(float2 a, float2 b) {
	return float2( a.x*b.x -  a.y*b.y,a.x*b.y + a.y * b.x);
}



float pattern(  float2 p, out float2 q, out float2 r )
{
	q.x = fbm( p  +0.00*time);
	q.y = fbm( p + float2(1.0, 1.0));
	
	r.x = fbm( p +1.0*q + float2(1.7,9.2)+0.15*time );
	r.y = fbm( p+ 1.0*q + float2(8.3,2.8)+0.126*time);
	//r = cMul(q,q+0.1*time);
	return fbm(p +1.0*r + 0.0* time);
}

  float3 color1 = float3(0.101961,0.619608,0.666667);
  float3 color2 = float3(0.666667,0.666667,0.498039);
  float3 color3 = float3(0,0,0.164706);
  float3 color4 = float3(0.666667,1,1);

float4 main(float2 gl_FragCoord: TEXCOORD0, float4 gl_FragColor: COLOR ) : COLOR {

                gl_FragCoord.y = 1 - gl_FragCoord.y; gl_FragCoord = gl_FragCoord * resolution;
                float2 q;
	float2 r;
	float2 c = 1000.0*gl_FragCoord.xy/ resolution.xy;
	float f = pattern(c*0.01,q,r);
	float3 col = lerp(color1,color2,clamp((f*f)*4.0,0.0,1.0));
	col = color2;
	col = lerp(col,color3,clamp(length(q),0.0,1.0));
	col = lerp(col,color4,clamp(length(r.x),0.0,1.0));
	gl_FragColor =  float4((0.2*f*f*f+0.6*f*f+0.5*f)*col,1.0);
            return gl_FragColor;

}

// Vertex Shader
struct VS_INPUT
{
    float3 position	: POSITION;
    float2 texture0     : TEXCOORD0;
};

struct VS_OUTPUT
{
     float4 hposition : POSITION;
     float2 texture0  : TEXCOORD0;
};

VS_OUTPUT myvs( VS_INPUT IN )
{
    VS_OUTPUT OUT;
    OUT.hposition = float4(IN.position.x ,IN.position.y ,IN.position.z, 1);
    OUT.texture0 = IN.texture0;
    return OUT;
}
// ---------------------------------------------
technique Start
{
    pass p1  
    {  VertexShader = compile vs_3_0 myvs();
        PixelShader = compile ps_3_0 main();     }
}




