float4x4 worldViewProjI; float postfx;

#ifdef GL_ES
// precision mediump float;
#endif

uniform float time;
uniform float2 mouse;
uniform float2 resolution = float2(933,511) ; // Screensize;


float map(float3 p)
{
      int MAX_ITER = 10;
      float BAILOUT=4.0;
    float Power=6.0;

    float3 v = p;
    float3 c = v;

    float r=0.0;
    float d=1.0;
    for(int n=0; n<=MAX_ITER; ++n)
    {
        r = length(v);
        if(r>BAILOUT) break;

        float theta = acos(v.z/r);
        float phi = atan2(v.y, v.x);
        d = pow(r,Power-1.2)*Power*d+1.0;

        float zr = pow(r,Power);
        theta = theta*Power;
        phi = phi*Power;
        v = (float3(sin(theta)*cos(phi), sin(phi)*sin(theta), cos(theta))*zr)+c;
    }
    return 0.5*log(r)*r/d;
}


float4 main(float2 gl_FragCoord: TEXCOORD0, float4 gl_FragColor: COLOR ) : COLOR
{
gl_FragCoord.y = 1 - gl_FragCoord.y; gl_FragCoord = gl_FragCoord * resolution;

    float2 pos = (gl_FragCoord.xy*2.0 - resolution.xy) / resolution.y;
    float3 camPos = float3(cos(time*0.3+time), sin(time*0.3-time), 2.-sin(time*.2)/2.);
    float3 camTarget = float3(0.0, 0.0, 0.0);

    float3 camDir = normalize(camTarget-camPos);
    float3 camUp  = normalize(float3(0.0, 1.0, 0.0));
    float3 camSide = cross(camDir, camUp);
    float focus = 1.8;

    float3 rayDir = normalize(camSide*pos.x + camUp*pos.y + camDir*focus);
    float3 ray = camPos;
    float m = 0.0;
    float d = 0.0, total_d = 0.0;
      int MAX_MARCH = 3000;
      float MAX_DISTANCE = 10000.0;
    for(int i=0; i<MAX_MARCH; ++i) {
        d = map(ray);
        total_d += d;
        ray += rayDir * d;
        m += 1.0;
        if(d<0.001) { break; }
        if(total_d>MAX_DISTANCE) { total_d=MAX_DISTANCE; break; }
    }

    float c = (total_d)*0.0001;
    float4 result = float4( 1.0-float3(c, c, c) - float3(0.025, 0.025, 0.02)*m*0.8, 1.0 );
    gl_FragColor = result;
            return gl_FragColor;

}
// Vertex Shader
struct VS_INPUT
{
    float3 position	: POSITION;
    float2 texture0     : TEXCOORD0;
};

struct VS_OUTPUT
{
     float4 hposition : POSITION;
     float2 texture0  : TEXCOORD0;
};

VS_OUTPUT myvs( VS_INPUT IN )
{
    VS_OUTPUT OUT;
    if (postfx == 0) {
      OUT.hposition = mul( worldViewProjI, float4(IN.position.x ,IN.position.y ,IN.position.z, 1) ); 
    } else {
      OUT.hposition = float4(IN.position.x ,IN.position.y ,IN.position.z, 1);
    }    
    OUT.texture0 = IN.texture0;
    return OUT;
}
// ---------------------------------------------
technique Start
{
    pass p1  
    {
        VertexShader = compile vs_3_0 myvs();
        PixelShader = compile ps_3_0 main();
    }
}


